module FLouvainSpec where

import Test.Hspec

import Protolude

-- FGL
import Data.Graph.Inductive

import Data.Graph.Clustering.FLouvain
import Data.Graph.Clustering.Louvain.Utils (mkFGraph, mkFGraph')
import Data.Graph.FGL

-- 1 -> 2 -> 3

simpleGraph :: FGraph () ()
simpleGraph = mkFGraph' [ (1, 2, 1.0)
                        , (2, 3, 1.0)
                        ]

simpleLGraph :: FGraph Text ()
simpleLGraph = mkFGraph [ (1, "one")
                        , (2, "two")
                        , (3, "three")]
                        [ (1, 2, 1.0)
                        , (2, 3, 1.0) ]

spec :: Spec
spec = do
  describe "FLouvain tests" $ do
    it "graphWeight computes correctly" $ do
      graphWeight simpleGraph `shouldBe` GraphWeightSum 2.0

    it "initialCgr computes correctly" $ do
      let cgr = initialCGr simpleGraph
          communities = lnodes cgr
      nodes cgr `shouldBe` [1, 2, 3]
      edges cgr `shouldBe` [ (1, 2)
                           , (2, 3) ]
      map comNodes communities `shouldBe` [[1], [2], [3]]

    it "replaceLNode works correctly" $ do
      let replaced = replaceLNode simpleLGraph (1, "ONE")
      nodes replaced `shouldBe` [1, 2, 3]
      lnodes replaced `shouldBe` ["ONE", "two", "three"]
